import "./styles.css";

let spread = new GC.Spread.Sheets.Workbook('ss', {sheetCount: 2});
initSpread(spread);

// initializes spread
function initSpread(spread) {
	let sheet = spread.getActiveSheet();

	addData(sheet);
	addData(spread.getSheet(1));

	let searchResults;
	// keeps track of previous selected cell and its style
	let details = {
		previousSheet: undefined,
		cellRow: undefined,
		cellCol: undefined,
		previousStyle: undefined
	};

	spread.bind(GC.Spread.Sheets.Events.EnterCell, (event, args) => {
		if (!details.cellRow || !details.cellCol) {
			return;
		}
		details.previousSheet.setStyle(details.cellRow, details.cellCol, details.previousStyle);
		details.cellRow = undefined;
		details.cellCol = undefined;
	})

	// highlights selected text
	document.getElementById('search-results').addEventListener('change', (event) => {
		let optionIndex = Number.parseInt(event.target.value);

		if (Number.isNaN(optionIndex)) {
			return;
		}

		if (details.cellRow && details.cellCol) {
			details.previousSheet.setStyle(details.cellRow, details.cellCol, details.previousStyle);
		}

		let selectedSearchResult = searchResults[optionIndex];
		let selectedSheet = spread.getSheetFromName(selectedSearchResult.sheetName);
		spread.setActiveSheetIndex(selectedSearchResult.foundSheetIndex);
		details.previousSheet = selectedSheet;

		// moves viewport to selected cell
		selectedSheet.showCell(selectedSearchResult.foundRowIndex, selectedSearchResult.foundColumnIndex, GC.Spread.Sheets.VerticalPosition.center, GC.Spread.Sheets.HorizontalPosition.center);

		highlightKeyword(selectedSheet, selectedSearchResult.foundRowIndex, selectedSearchResult.foundColumnIndex, details);
	})

	// initiates search
	document.getElementById('search-button').addEventListener('click', (e) => {
		let searchString = document.getElementById('search-input').value;

		if (!searchString) {
			return;
		}

		// search condition to find the search string in the workbook
		let searchCondition = new GC.Spread.Sheets.Search.SearchCondition();
		searchCondition.sheetArea = GC.Spread.Sheets.SheetArea.viewport;
		searchCondition.searchOrder = GC.Spread.Sheets.Search.SearchOrder.zOrder;
		searchCondition.searchFlags = GC.Spread.Sheets.Search.SearchFlags.ignoreCase;
		searchCondition.searchTarget = GC.Spread.Sheets.Search.SearchFoundFlags.cellText;
		searchCondition.searchString = searchString;

		searchResults = searchKeywordInWorkbook(spread, searchCondition);
		addOptions(searchResults);
	});
}

// highlights selected keyword
function highlightKeyword(sheet, row, col, details) {
	details.previousStyle = sheet.getStyle(row, col);

	details.cellRow = row;
	details.cellCol = col;

	let newStyle;
	
	if (details.previousStyle) {
		newStyle = details.previousStyle.clone();
	} else {
		newStyle = new GC.Spread.Sheets.Style();
	}

	if (newStyle.backColor && newStyle.backColor.toLowerCase() === 'yellow') {
		newStyle.backColor = 'green';
	} else {
		newStyle.backColor = 'yellow';
	}
	
	sheet.setStyle(row, col, newStyle);
	sheet.setActiveCell(row, col);
}

// adds option to select
function addOptions(searchResults) {
	let selectElement = document.getElementById('search-results');
	selectElement.textContent = '';
	let defaultOption = document.createElement('option');
	defaultOption.textContent = 'Select';
	defaultOption.value = 'Select';
	selectElement.appendChild(defaultOption);

	let index = 0;
	for (let result of searchResults) {
		let optionElement = document.createElement('option');
		let range = new GC.Spread.Sheets.Range(result.foundRowIndex, result.foundColumnIndex, 1, 1);
		let cellReference = GC.Spread.Sheets.CalcEngine.rangeToFormula(range);

		optionElement.textContent = `Sheet:${result.sheetName}    Cell:${cellReference}`;
		optionElement.value = index;
		selectElement.appendChild(optionElement);
		index++;
	}
}

// searches a keyword in workbook
function searchKeywordInWorkbook(spread, searchCondition) {
	let resultArray = [];
	
	// searches in every sheet
	for (let sheet of spread.sheets) {
		let sheetIndex = spread.getSheetIndex(sheet.name());

		searchCondition.startSheetIndex = sheetIndex;
		searchCondition.endSheetIndex = sheetIndex;
		searchCondition.rowStart = 0;
		searchCondition.columnStart = 0;
		searchCondition.rowEnd = sheet.getRowCount() - 1;
		searchCondition.columnEnd = sheet.getColumnCount() - 1;

		searchKeywordInWorksheet(sheet, sheetIndex, searchCondition, resultArray);
	}

	return resultArray;
}

// searches a keyword in worksheet
function searchKeywordInWorksheet(sheet, sheetIndex, searchCondition, resultArray) {

	let searchResult = sheet.search(searchCondition);

	if (!searchResult.foundString) {
		return;
	}

	searchResult.foundSheetIndex = sheetIndex;
	searchResult.sheetName = sheet.name();
	resultArray.push(searchResult);

	searchCondition.rowStart = searchResult.foundRowIndex;
	searchCondition.columnStart = searchResult.foundColumnIndex + 1;

	searchKeywordInWorksheet(sheet, sheetIndex, searchCondition, resultArray);
}

// adds data
function addData(sheet) {
	sheet.suspendPaint();
	let list = ['apple', 'mango', 'hello', 'banana'];

	sheet.setValue(2, 2, 'Enter search string in input box and click search button');
	sheet.setValue(3, 2, 'Search results are in dropdown of select');
	sheet.setValue(4, 2, 'click any search result to go to that cell');
	sheet.setValue(4, 2, 'Import your own file to search for keyword');

	let style = new GC.Spread.Sheets.Style();
	style.foreColor = 'red';

	for (let row = 6; row < 10; row++) {
		for (let col = 3; col < 6; col++) {
			let randomValue = Math.floor((Math.random() * 3)) + 1;
			sheet.setValue(row, col, list[randomValue]);
			sheet.setStyle(row, col, style);
		}
	}
	sheet.resumePaint();
}

// imports excel file
function importExcel(spread, file) {
	let excelIO = new GC.Spread.Excel.IO();
	excelIO.open(file, (json) => {
		spread.fromJSON(json);
	});
}
